/*
 * EnumValue.java
 * Part of the B'am library - a collection of general-purpose utility classes.
 * 
 * Copyright (C)2006 Stephen Battey
 * Contact: resource@lucidviews.net
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

package net.lucidviews.util;


/**
 * Base-class for defining an enumeration type.
 * <br />
 * <br />
 * To define an enumeration extend this class, implementing the
 * <code>getEnumeration</code> method, and create instances for each value in
 * your enumeration.
 * <br />
 * For example:
 * <code>
 * <br />public class DistanceUnit extends EnumValue
 * <br />{
 * <br />&nbsp;&nbsp;// create a list for the enumeration values
 * <br />&nbsp;&nbsp;public static final Enumeration ENUMERATION = new Enumeration();
 * <br />&nbsp;&nbsp;
 * <br />&nbsp;&nbsp;// provide the EnumValue base class with access to the enumeration
 * <br />&nbsp;&nbsp;public Enumeration getEnumeration()
 * <br />&nbsp;&nbsp;{
 * <br />&nbsp;&nbsp;&nbsp;&nbsp;return ENUMERATION;
 * <br />&nbsp;&nbsp;}
 * <br />&nbsp;&nbsp;
 * <br />&nbsp;&nbsp;// define the enumeration values
 * <br />&nbsp;&nbsp;public static final DistanceUnit METRE = new DistanceUnit();
 * <br />&nbsp;&nbsp;public static final DistanceUnit YARD = new DistanceUnit();
 * <br />&nbsp;&nbsp;public static final DistanceUnit KILOMETRE = new DistanceUnit();
 * <br />&nbsp;&nbsp;public static final DistanceUnit MILE = new DistanceUnit();
 * <br />&nbsp;&nbsp;
 * <br />&nbsp;&nbsp;// make the constructor private to protect this enum
 * <br />&nbsp;&nbsp;private DistanceUnit()
 * <br />&nbsp;&nbsp;{
 * <br />&nbsp;&nbsp;}
 * <br />}
 * </code>
 * <br />
 * It is essential that all constructors of the class are <code>protected</code>
 * or <code>private</code> so that no other instances can be created, which
 * would extend your enumeration with values that you did not define. This
 * includes the default constructor, which must be declared
 * <code>protected</code> or <code>private</code> if you define no other
 * constructor.
 * <br />
 * However, it is recommended that you assign a user-friendly text string to
 * each of your enumeration values. To do this you will need to define a
 * constructor in your enum class like this:
 * <code>
 * <br />&nbsp;&nbsp;...
 * <br />&nbsp;&nbsp;// define the enumeration values
 * <br />&nbsp;&nbsp;public static final DistanceUnit METRE = new DistanceUnit( "metre" );
 * <br />&nbsp;&nbsp;public static final DistanceUnit YARD = new DistanceUnit( "yard" );
 * <br />&nbsp;&nbsp;public static final DistanceUnit KILOMETRE = new DistanceUnit( "kilometre" );
 * <br />&nbsp;&nbsp;public static final DistanceUnit MILE = new DistanceUnit( "mile" );
 * <br />&nbsp;&nbsp;
 * <br />&nbsp;&nbsp;// make the constructor private to protect this enum
 * <br />&nbsp;&nbsp;private DistanceUnit( String unitName )
 * <br />&nbsp;&nbsp;{
 * <br />&nbsp;&nbsp;&nbsp;&nbsp;super( unitName );
 * <br />&nbsp;&nbsp;}
 * <br />&nbsp;&nbsp;...
 * </code>
 * <br />
 * <br />
 * The enumeration type is now ready to use. Using the enumeration type makes
 * the source code more readable and ensures the compiler performs some type
 * checking on parameter values.
 * <br />
 * For example, the method:
 * <br /><code>public void setDistanceUnit( int distanceUnit )</code>
 * <br />would become
 * <br /><code>public void setDistanceUnit( DistanceUnit newValue )</code>
 * <br />
 * The primary advantage is that the <code>newValue</code> must be one of the
 * constants defined by the <code>DistanceUnit</code> class - in the first
 * interface, the <code>distanceUnit</code> could be any <code>int</code> value
 * and the method would need to perform some range checks first and throw an
 * exception. With enumerations the checks are done at compile time and no
 * exception needs to be caught.
 * <br />
 * <br />
 * Enumeration values can be taken from the exported constants in the
 * <code>EnumValue</code> sub class or derived from their index or user-friendly
 * name.
 * <br />
 * For example, if a string value "<code>mile</code>" is read from a
 * configuration file, the enumeration value can be looked-up:
 * <code>
 * <br />String distanceUnitName = ...;
 * <br />EnumValue distanceUnit = DistanceUnit.ENUMERATION.lookupValue( distanceUnitName );
 * </code>
 * <br />
 * <br />
 * <b>IMPORTANT</b>: use the <code>==</code> operator, not the
 * <code>equals</code> method to evaluate or compare <code>EnumValue</code>s.
 * <br />
 * The <code>equals</code> method performs an internal representation
 * comparision and is provided so that string and integer values will match
 * the text-value and index of an enumeration value.
 * <br />
 * Each enumeration value is a new, unique constant and should be compared
 * directly. For example:
 * <code>
 * <br />String distanceUnitName = ...;
 * <br />EnumValue distanceUnit = DistanceUnit.ENUMERATION.lookupValue( distanceUnitName );
 * <br />if (distanceUnit == DistanceUnit.MILE)
 * <br />{
 * <br />&nbsp;&nbsp;...
 * <br />}
 * </code>
 * <br />
 * <br />
 * Note: Java version 1.5 contains support for enum types. These classes
 * (<code>Enumeration</code> and <code>EnumValue</code>) are provided to give
 * backwards compatibiity with previous versions of Java.
 * <br />
 * <br />
 * @see net.lucidviews.util.Enumeration
 * @author Stephen Battey
 * @version $Revision: 1.3 $
 * @since 1.0
 */
public abstract class EnumValue implements java.io.Serializable
{

	/*
	 * Local constants
	 */

	/** A value assigned to the index variable before the index of the enum
	 *  value is known. */
	protected static int UNDEFINED_INDEX = -1;


	/*
	 * Instance variables
	 */

	/** The user-friendly name associated with this enumeration value. */
	protected String  _textValue;

	/** A flag indicating if the text value is case sensitive. */
	protected boolean _isCaseSensitive;

	/** The index of this value within the enumeration. */
	protected int _index;


	/*
	 * Constructor(s)
	 */

	/**
	 * Create an enumeration value that can be identified by the given text.
	 * <br />
	 * <br />
	 * @param textValue the <code>String</code> that is associated with this
	 *                  enum value
	 * @param isCaseSensitive indicates if a <code>String</code> should be
	 *                        associated with this enum value only if the case
	 *                        matches that in the text value
	 */
	protected EnumValue( String textValue, boolean isCaseSensitive )
	{
		_textValue = textValue;
		_isCaseSensitive = isCaseSensitive;
		_index = UNDEFINED_INDEX;
		/* it is important to set the index as undefined first, otherwise
		   the index will default to 0, which will cause this enum value to
		   be equal to any enum value at index 0 of the enumeration. */

		_index = getEnumeration().addValue( this );
	}

	/**
	 * Create an enumeration value that can be identified by the given text.
	 * <br />
	 * The text will not be treated as case sensitive.
	 * <br />
	 * <br />
	 * @param textValue the <code>String</code> that is associated with this
	 *                  enum value
	 */
	protected EnumValue( String textValue )
	{
		this( textValue, false );
	}

	/**
	 * Create an enumeration value with no user-friendly name.
	 * <br />
	 * The value can then only be cross-referenced by its index.
	 */
	protected EnumValue()
	{
		this( null, false );
	}


	/*
	 * Abstract method(s)
	 */

	/**
	 * Obtain the enumeration this value belongs to.
	 * <br />
	 * See the class JavaDoc, at the top of this page, for information on how to
	 * implement this method correctly.
	 * <br />
	 * <br />
	 * @return a list of enumeration values that this value is part of
	 */
	public abstract Enumeration getEnumeration();


	/*
	 * Overridden methods of Object base class
	 */

	public String toString()
	{
		if (_textValue != null)
			return _textValue;
		else
			return String.valueOf( _index );
	}

	public int hashCode()
	{
		/* Always use the toString method to derive a hash-code.
		   Using the _index could lead to matching the hash code of the index
		   of one enum value to the hash code of the text-value of another enum
		   value. */
		String hashText;

		if (_isCaseSensitive)
			hashText = toString();
		else
			hashText = toString().toLowerCase();

		return hashText.hashCode();
	}

	public boolean equals( Object obj )
	{
		/* for an EnumValue object, the values must be the same instance */
		if (obj instanceof EnumValue)
			return obj == this;

		/* for other types of object, compare the string representations
		   - this will ensure we match Strings and Numbers */
		String objString = obj.toString();

		/* try comparing the user-friendly name */
		if (_textValue != null)
		{
			if (_isCaseSensitive)
			{
				if (objString.equals( _textValue ))
					return true;
			}
			else
			{
				if (objString.equalsIgnoreCase( _textValue ))
					return true;
			}
		}

		/* failing that, try comparing the index */
		if (_index != UNDEFINED_INDEX)
		{
			String indexString = Integer.toString( _index );
			if (objString.equals( indexString ))
				return true;
		}

		return false;
	}

}


/*
 * Change History
 *
 * $Log: EnumValue.java,v $
 * Revision 1.3  2006/09/06 20:52:02  Steve
 * Added legal disclaimer.
 *
 * Revision 1.2  2006/06/12 20:50:16  Steve
 * Fixed JavaDoc x-ref.
 *
 * Revision 1.1  2006/04/17 15:33:38  Steve
 * Initial revision.
 *
 */
