/*
 * Enumeration.java
 * Part of the B'am library - a collection of general-purpose utility classes.
 * 
 * Copyright (C)2006 Stephen Battey
 * Contact: resource@lucidviews.net
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

package net.lucidviews.util;

import java.util.NoSuchElementException;
import java.util.ArrayList;
import java.util.ListIterator;
import java.util.Iterator;


/**
 * An enumerated list of values.
 * <br />
 * An enumeration is a finite set of distinct values.
 * <br />
 * Each value in an <code>Enumeration</code> is an instance of an
 * <code>EnumValue</code>. Creating an enumeration simply requires you to
 * create a sub-class of <code>EnumValue</code> - see the documentation of that
 * class for more information.
 * <br />
 * <br />
 * Note: Java version 1.5 contains support for enum types. These classes
 * (<code>Enumeration</code> and <code>EnumValue</code>) are provided to give
 * backwards compatibiity with previous versions of Java.
 * <br />
 * <br />
 * @see net.lucidviews.util.EnumValue
 * @author Stephen Battey
 * @version $Revision: 1.4 $
 * @since 1.0
 */
public class Enumeration
{
	
	/*
	 * Instance variables
	 */
	
	/** The values in this enumeration. */
	protected ArrayList _values;
	
	/** The type of enumeration values. */
	protected Class _enumType;
	
	
	/*
	 * Constructor(s)
	 */
	
	/**
	 * Create an enumeration, specifying the number of values so that the
	 * enumeration list can be sized at creation, thereby improving performance.
	 * <br />
	 * <br />
	 * @param initialCapacity the expected number of values in this enumeration
	 */
	public Enumeration( int initialCapacity )
	{
		_values = new ArrayList( initialCapacity );
		_enumType = null;
	}
	
	/**
	 * Create an enumeration.
	 */
	public Enumeration()
	{
		_values = new ArrayList();
		_enumType = null;
	}
	
	
	/*
	 * Methods
	 */
	
	/**
	 * Add a value to this enumeration.
	 * <br />
	 * An enumeration may only hold values that are of the same type - i.e.
	 * instances of the same <code>EnumValue</code> class. Attempting to add a
	 * different type of value to the enumeration will result in an exception.
	 * <br />
	 * All values must be unique. Attempting to add a duplicate value will also
	 * result in an exception.
	 * <br />
	 * <br />
	 * <code>Enumeration</code>s are not editable once they have been created.
	 * This method should only be called from within the <code>EnumValue</code>
	 * class while the list of enumeration values is being built. Therefore this
	 * method has been given package scope to prevent the list being changed
	 * after it has been created.
	 * <br />
	 * <br />
	 * @param value enumeration value to be added to this list
	 * @return the index of where the value is stored in the enumeration list
	 * @throws IllegalStateException invalid value for this list
	 */
	int addValue( EnumValue value ) throws IllegalStateException
	{
		if (_enumType == null)
		{
			/* remember the type of value being used in this list */
			_enumType = value.getClass();
		}
		else
		{
			/* make sure the values are all of the same type */
			if (!value.getClass().equals( _enumType ))
				throw new IllegalStateException( "Cannot add enumeration value '" + value + "' of type " + value.getClass().getName() + " to enuemration list holding values of type " + _enumType.getName() + "; enumeration values must all be of the same type." );
		}
		
		/* guard against duplicate elements */
		if (_values.contains( value ))
			throw new IllegalStateException( "Cannot allow duplicate enumeration value '" + value + "' of type " + _enumType.getName() + "." );
		
		/* now store the value */
		if (!_values.add( value ))
			throw new IllegalStateException( "Addition of enumeration value '" + value + "' to enumeration of type " + _enumType.getName() + " failed." );
		
		return _values.indexOf( value );
	}
	
	/**
	 * Lookup a value in this enumeration.
	 * <br />
	 * The <code>value</code> parameter can be any <code>Object</code>. The
	 * return value is always an <code>EnumValue</code> that is considered equal
	 * (using the <code>equals</code> method to compare values) to the specified
	 * <code>value</code>.
	 * <br />
	 * Thus, if the <code>Enumeration</code> contains an <code>EnumValue</code>
	 * with id 6 and string-value "yard":
	 * <br /><code>lookupValue( "yard" );</code>
	 * <br /><code>lookupValue( new Integer( 6 ) );</code>
	 * <br />
	 * will both return the <code>EnumValue</code> instance.
	 * <br />
	 * <br />
	 * @param value the value to find
	 * @return the matching enumeration value
	 * @throws NoSuchElementException the value could not be found
	 */
	public EnumValue lookupValue( Object value ) throws NoSuchElementException
	{
		Iterator enumValueItr = _values.iterator();
		while (enumValueItr.hasNext())
		{
			EnumValue enumValue = (EnumValue)enumValueItr.next();
			
			if (enumValue.equals( value ))
			// NB: it is critical to use 'enumValue.equals( value )' and not
			//     'value.equals( enumValue )', since the latter will almost
			//     always be false, but the EnumValue object may allow
			//     comparison with a String and find an equality
				return enumValue;
		}
		
		throw new NoSuchElementException( "No enumeration value '" + value + "' of type " + _enumType.getName() + " exists (enum list: " + _values + ")." );
	}
	
	
	/*
	 * Access methods
	 */
	
	/**
	 * Returns the type of enum value being stored in this enumeration.
	 * <br />
	 * This will return <code>null</code> if no values have been added to this
	 * enumeration list.
	 * <br />
	 * <br />
	 * @return the type of this enumeration
	 */
	public Class getType()
	{
		return _enumType;
	}
	
	/**
	 * Returns the number of values in this enumeration.
	 * <br />
	 * <br />
	 * @return the number of values in this enumeration
	 */
	public int size()
	{
		return _values.size();
    }
	
	/**
	 * Tests if this enumeration has no values.
	 * <br />
	 * <br />
	 * @return <code>true</code> if no values have been defined in this
	 *         enuemration; <code>false</code> otherwise
	 */
	public boolean isEmpty()
	{
		return _values.isEmpty();
	}
	
	/**
	 * Returns an array containing all of the values in this enumeration in the
	 * correct order.
	 * <br />
	 * <br />
	 * @return an array containing all of the elements in this list in the
	 *         correct order
	 */
	public Object[] toArray()
	{
		return _values.toArray();
	}

	/**
	 * Returns an iterator of the values in this enumeration (in proper
	 * sequence).
	 * <br />
	 * <br />
	 * @return an iterator of the enumeration values (<codeEnumValue</code>s)
	 * @see net.lucidviews.util.EnumValue
	 */
	public ListIterator listIterator()
	{
		return _values.listIterator();
	}
	
	
	/*
	 * Overridden methods of Object base class
	 */
	
	public String toString()
	{
		StringBuffer buf = new StringBuffer();

		buf.append( "Enumeration[enumType:" );
		buf.append( _enumType.getName() );
		buf.append( ",values" );
		buf.append( _values );
		buf.append( "]" );

		return buf.toString();
	}
	
	public int hashCode()
	{
		return toString().hashCode();
	}
	
	public boolean equals( Object obj )
	{
		if (obj instanceof Enumeration)
		{
			Enumeration enumObj = (Enumeration)obj;

			/* all attributes must be equal (NB _enumType could be null) */
			return (   (_enumType == enumObj._enumType)
			        && (_values.equals( enumObj._values ))  );
		}
		else
		{
			return false;
		}
	}
	
}


/*
 * Change History
 *
 * $Log: Enumeration.java,v $
 * Revision 1.4  2006/09/06 20:52:02  Steve
 * Added legal disclaimer.
 *
 * Revision 1.3  2006/08/02 19:55:59  Steve
 * Added more access methods: size, isEmpty, toArray, listIterator.
 *
 * Revision 1.2  2006/04/27 21:05:14  Steve
 * Added getType method.
 *
 * Revision 1.1  2006/04/17 15:33:38  Steve
 * Initial revision.
 *
 */
