/*
 * PropertyException.java
 * Part of the B'am library - a collection of general-purpose utility classes.
 * 
 * Copyright (C)2006 Stephen Battey
 * Contact: resource@lucidviews.net
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

package net.lucidviews.util;

import net.lucidviews.util.text.ParseValueException;


/**
 * Exception raised by the <code>Properties</code> class when a property is
 * missing or invalid.
 * <br />
 * The exception stores extra information about the source of the properties
 * table, the id/key of the property and its value.
 * <br />
 * <br />
 * @see net.lucidviews.util.Properties
 * @author Stephen Battey
 * @version $Revision: 1.3 $
 * @since 1.0
 */
public class PropertyException extends Exception
{

	/*
	 * Exported constants
	 */

	/** Property value used when the error is caused by a missing value. */
	public static final String MISSING_VALUE = "<null value>";


	/*
	 * Instance variables
	 */

	/** A description of where the properties were read from. */
	protected String _propertySource;

	/** The key that references the property that was missing/invalid. */
	protected String _propertyKey;

	/** The value of the property that was not valid. */
	protected String _propertyValue;


	/*
	 * Constructor(s)
	 */

	/**
	 * Create a new exception caused by an invalid property value.
	 * <br />
	 * <br />
	 * @param propertySource a description of where the property was read from
	 * @param propertyKey the key that references the property in the
	 *                    <code>Properties</code> table
	 * @param propertyValue the value that was invalid
	 * @param parseException the error raised when trying to parse the property
	 *                       value
	 */
	public PropertyException( String propertySource, String propertyKey, String propertyValue, ParseValueException parseException )
	{
		super( "Error in " + propertySource + ". Property: " + propertyKey + "; value: " + propertyValue + ". " + parseException.getMessage(), parseException );

		_propertySource = propertySource;
		_propertyKey = propertyKey;
		_propertyValue = propertyValue;
	}

	/**
	 * Create a new exception caused by a missing property value.
	 * <br />
	 * <br />
	 * @param propertySource a description of where the property was expected to
	 *                       be defined
	 * @param propertyKey the key that was expected to reference the property in
	 *                    the <code>Properties</code> table
	 */
	public PropertyException( String propertySource, String propertyKey )
	{
		super( "Error in " + propertySource + ". Property: " + propertyKey + " was not defined." );

		_propertySource = propertySource;
		_propertyKey = propertyKey;
		_propertyValue = MISSING_VALUE;
	}


	/*
	 * Access methods
	 */

	/**
	 * @return a description of where the properties were read from
	 */
	public String getPropertySource()
	{
		return _propertySource;
	}

	/**
	 * @return the key that references the property that was missing/invalid
	 */
	public String getPropertyKey()
	{
		return _propertyKey;
	}

	/**
	 * @return the invalid property value (<code>null</code> if the value did
	 *         not exist)
	 */
	public String getPropertyValue()
	{
		return _propertyValue;
	}

}


/*
 * Change History
 *
 * $Log: PropertyException.java,v $
 * Revision 1.3  2006/09/06 20:52:02  Steve
 * Added legal disclaimer.
 *
 * Revision 1.2  2006/06/12 20:50:44  Steve
 * Removed out-of-date JavaDoc x-refs.
 *
 * Revision 1.1  2006/05/14 16:35:04  Steve
 * Initial revision.
 *
 */
