/*
 * FileComparator.java
 * Part of the B'am library - a collection of general-purpose utility classes.
 * 
 * Copyright (C)2006 Stephen Battey
 * Contact: resource@lucidviews.net
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

package net.lucidviews.util.io;

import java.io.File;
import java.util.Comparator;


/**
 * An extension to the <code>Comparator</code> interface implemented by classes
 * that compare files.
 * <br />
 * <br />
 * To implement this interface you will need to implement a
 * <code>compare</code> method for comparing file objects as well as the
 * standard <code>compare</code> and <code>equals</code> methods defined in the
 * <code>java.util.Comparator</code> interface.
 * <br />
 * Classes may prefer to extend the <code>AbstractFileComparator</code> class,
 * which implements the <code>compare( Object, Object )</code> method,
 * automatically routing calls to the <code>compare( File, File )</code> method.
 * <br />
 * This reduces the number of methods that need to be implemented.
 * <br />
 * <br />
 * @author Stephen Battey
 * @version $Revision: 1.1 $
 * @since 1.0
 * @see java.util.Comparator
 */
public interface FileComparator extends Comparator
{
	
	/**
	 * Compares two <code>File</code>s for order. Returns a negative integer,
	 * zero, or a positive integer as the first file is less than, equal
	 * to, or greater than the second.
	 * <br />
	 * <br />
	 * The implementor must ensure that <tt>sgn(compare(x, y)) ==
	 * -sgn(compare(y, x))</tt> for all <tt>x</tt> and <tt>y</tt>. (This
	 * implies that <tt>compare(x, y)</tt> must throw an exception if and only
	 * if <tt>compare(y, x)</tt> throws an exception.)
	 * <br />
	 * <br />
	 * The implementor must also ensure that the relation is transitive:
	 * <tt>((compare(x, y)&gt;0) &amp;&amp; (compare(y, z)&gt;0))</tt> implies
	 * <tt>compare(x, z)&gt;0</tt>.
	 * <br />
	 * <br />
	 * Finally, the implementer must ensure that <tt>compare(x, y)==0</tt>
	 * implies that <tt>sgn(compare(x, z))==sgn(compare(y, z))</tt> for all
	 * <tt>z</tt>.
	 * <br />
	 * <br />
	 * It is generally the case, but <i>not</i> strictly required that 
	 * <tt>(compare(x, y)==0) == (x.equals(y))</tt>.  Generally speaking,
	 * any comparator that violates this condition should clearly indicate
	 * this fact.  The recommended language is "Note: this comparator
	 * imposes orderings that are inconsistent with equals."
	 * <br />
	 * <br />
	 * @param file1 the first file to be compared
	 * @param file2 the second file to be compared
	 * @return a negative integer, zero, or a positive integer as the
	 * 	       first file is less than, equal to, or greater than the
	 *	       second
	 */
	int compare( File file1, File file2 );
	
}


/*
 * Change History
 *
 * $Log: FileComparator.java,v $
 * Revision 1.1  2006/09/13 20:09:30  Steve
 * Initial revision.
 *
 */