/*
 * FreshRollingFileAppender.java
 * Part of the B'am library - a collection of general-purpose utility classes.
 * 
 * Copyright (C)2006 Stephen Battey
 * Contact: resource@lucidviews.net
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

package net.lucidviews.util.log4j;

import org.apache.log4j.RollingFileAppender;
import org.apache.log4j.spi.LoggingEvent;


/**
 * <code>FreshRollingFileAppender</code> extends
 * <code>RollingFileAppender</code> inheriting and retaining all the standard
 * file backup functionality while ensuring a fresh log file is created each
 * time the application is started.
 * <br />
 * <br />
 * @author Stephen Battey
 * @version $Revision: 1.2 $
 * @since 1.0
 * @see org.apache.log4j.RollingFileAppender
 */
public class FreshRollingFileAppender extends RollingFileAppender
{
	
	// TODO Would like to make this class simply implement the Appender
	//      interface and act as a wrapper around a RollingFileAppender so
	//      that it can be applied to sub-classes of RollingFileAppender.
	//      However, if appenders are loaded like beans we'd need member
	//      variables for each property of RollingFileAppender and any
	//      sub-class.
	
	/*
	 * Instance variables
	 */
	
	/** Flag indicating if a new log file should be created for this instance of
	 *  the application. */
	protected boolean freshLogAtStartup = true;
	
	/** Flag set just until the first logging event occurs. */
	private boolean firstLoggingEvent = true;
	
	
	/*
	 * Constructor(s)
	 */
	
	/**
	 * The default constructor.
	 */
	public FreshRollingFileAppender()
	{
		super();
	}
	
	
	/*
	 * Overridden methods
	 */
	
	protected void subAppend( LoggingEvent event )
	// The subAppend method gets called once it has been determined the log is
	// open for more records and the event is of sufficient priority to be
	// included in the log.
	{
		if (this.firstLoggingEvent)
		{
			if (this.freshLogAtStartup)
			{
				this.rollOver();
			}
			this.firstLoggingEvent = false;
		}
		
		super.subAppend( event );
	}
	
	
	/*
	 * Access methods
	 */
	
	/**
	 * Check if this appender has created a fresh log file when the application
	 * started.
	 * <br />
	 * <br />
	 * @return <code>true</code> if a new log file was created at startup
	 */
	public boolean isFreshLogAtStartup()
	{
		return this.freshLogAtStartup;
	}
	
	/**
	 * Tell this appender whether to create a fresh log file when the
	 * application starts.
	 * <br />
	 * <br />
	 * @param freshLogAtStartup <code>true</code> if a new log file should be
	 *                          created at startup
	 */
	public void setFreshLogAtStartup( boolean freshLogAtStartup )
	{
		this.freshLogAtStartup = freshLogAtStartup;
	}
	
}


/*
 * Change History
 *
 * $Log: FreshRollingFileAppender.java,v $
 * Revision 1.2  2006/09/06 20:52:02  Steve
 * Added legal disclaimer.
 *
 * Revision 1.1  2006/08/26 16:50:53  Steve
 * Initial revision.
 *
 */
